%TEST_BLG Test program for the BLGAUSSEXP function.
%   This script computes all communicabilities between some nodes of two
%   undirected test networks. The results obtained by block Gauss/Antigauss
%   quadrature are compared to those obtained by expm.

%   Caterina Fenu & Giuseppe Rodriguez, University of Cagliari, Italy
%   David Martin & Lothar Reichel, Kent State University, U.S.A.
%   Email: kate.fenu@gmail.com, rodriguez@unica.it, 
%          dmarti49@kent.edu, reichel@math.kent.edu
%
%   Last revised August 4, 2013

% test matrices
testmats = {
	'autobahn'
	'yeast'
	};
nmatr = size(testmats,1);

clc
fprintf('\nComputation of communicabilities.\n\n')
for i = 1:nmatr
	fprintf('%3d - %s\n', i, testmats{i})
end
fprintf('\nPress enter for default values.\n')
tst = input('choose a test matrix (def. 1): ');
if isempty(tst), tst = 1; end
load(sprintf('adjacency_%s',testmats{tst}))
dn = full(~any(A));
if any(dn)
	fprintf('purging %d dangling nodes.\n', sum(dn))
	A = A(~dn,~dn);
end

n = size(A,1);
fprintf('number of nodes: %d\n',n)
numnod = input('choose the number of nodes to be identified (def. 5): ');
if isempty(numnod), numnod = 5; end

matrixname = testmats{tst};

% options
clear opts
opts.gausstolq = 1e-3;		% quadrature accuracy
opts.sgcshift = 1;		% spectrum shift
opts.gaussreorth = 0;		% reorthogonalization

fprintf('Computing communicabilities by expm ')
tic
E = expm(A);
allx = E(1:numnod,1:numnod);
sc = diag(allx);
fprintf('(%.2f seconds)\n',toc)

fprintf('Computing communicabilities by block Gauss-Lanczos ')
tic
W = [eye(numnod); zeros(n-numnod,numnod)];
W = [W ones(n,1)];
[fval info_bl] = blgaussexp(A,W,opts);
fprintf('(%.2f seconds)\n',toc)
fval = fval(1:numnod,1:numnod);

format short e

fprintf('\nResults: \n')
disp(fval)
fprintf('Relative errors with respect to expm: \n')
disp(abs(allx-fval)./(allx+(allx<sqrt(eps))))

